This is the mail archive of the binutils@sourceware.org mailing list for the binutils project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

Add "supported targets" to gold --help output


Both config.guess and libtool look for the literal string "supported
targets" in the output of ld --help.  This patch changes gold to
produce a string in the format that they expect.  I've verified that
this fixes config.guess; I haven't checked libtool.

Along the way I cleaned up the Target_selector interface so that there
is less duplication in the target files.  Typical targets should now
have a simple Target_selector.

Ian


2008-03-25  Ian Lance Taylor  <iant@google.com>

	* options.cc (options::help): Print list of supported targets.
	* target-select.h: Include <vector>.
	(class Target_selector): Make machine_, size_, and is_big_endian_
	fields const.  Add bfd_name_ and instantiated_target_ fields.
	(Target_selector::Target_selector): Add bfd_name parameter.
	(Target_selector::recognize): Make non-virtual, call
	do_recognize.
	(Target_selector::recognize_by_name): Make non-virtual, call
	do_recognize_by_name.
	(Target_selector::supported_names): New function.
	(Target_selector::bfd_name): New function.
	(Target_selector::do_instantiate_target): New pure virtual
	function.
	(Target_selector::do_recognize): New virtual function.
	(Target_selector::do_recognize_by_name): New virtual function.
	(Target_selector::instantiate_target): New private function.
	(supported_target_names): Declare.
	* target-select.cc (Target_selector::Target_selector): Update for
	new parameter and fields.
	(select_target_by_name): Check that the name matches before
	calling recognize_by_name.
	(supported_target_names): New function.
	* i386.cc (class Target_selector_i386): Update Target_selector
	constructor call.  Remove recognize and recognize_by_name.  Add
	do_instantiate_target.
	* x86_64.cc (class Target_selector_x86_64): Likewise.
	* testsuite/testfile.cc (class Target_selector_test): Update for
	changes to Target_selector.


Index: i386.cc
===================================================================
RCS file: /cvs/src/src/gold/i386.cc,v
retrieving revision 1.65
diff -p -u -r1.65 i386.cc
--- i386.cc	25 Mar 2008 18:37:16 -0000	1.65
+++ i386.cc	25 Mar 2008 21:14:53 -0000
@@ -2444,40 +2444,14 @@ class Target_selector_i386 : public Targ
 {
 public:
   Target_selector_i386()
-    : Target_selector(elfcpp::EM_386, 32, false)
+    : Target_selector(elfcpp::EM_386, 32, false, "elf32-i386")
   { }
 
   Target*
-  recognize(int machine, int osabi, int abiversion);
-
-  Target*
-  recognize_by_name(const char* name);
-
- private:
-  Target_i386* target_;
+  do_instantiate_target()
+  { return new Target_i386(); }
 };
 
-// Recognize an i386 object file when we already know that the machine
-// number is EM_386.
-
-Target*
-Target_selector_i386::recognize(int, int, int)
-{
-  if (this->target_ == NULL)
-    this->target_ = new Target_i386();
-  return this->target_;
-}
-
-Target*
-Target_selector_i386::recognize_by_name(const char* name)
-{
-  if (strcmp(name, "elf32-i386") != 0)
-    return NULL;
-  if (this->target_ == NULL)
-    this->target_ = new Target_i386();
-  return this->target_;
-}
-
 Target_selector_i386 target_selector_i386;
 
 } // End anonymous namespace.
Index: options.cc
===================================================================
RCS file: /cvs/src/src/gold/options.cc,v
retrieving revision 1.64
diff -p -u -r1.64 options.cc
--- options.cc	24 Mar 2008 03:48:29 -0000	1.64
+++ options.cc	25 Mar 2008 21:14:53 -0000
@@ -142,6 +142,17 @@ help()
   std::vector<const One_option*>::const_iterator it;
   for (it = registered_options.begin(); it != registered_options.end(); ++it)
     (*it)->print();
+
+  // config.guess and libtool.m4 look in ld --help output for the
+  // string "supported targets".
+  printf(_("%s: supported targets:"), gold::program_name);
+  std::vector<const char*> supported_names;
+  gold::supported_target_names(&supported_names);
+  for (std::vector<const char*>::const_iterator p = supported_names.begin();
+       p != supported_names.end();
+       ++p)
+    printf(" %s", *p);
+  printf("\n");
 }
 
 // For bool, arg will be NULL (boolean options take no argument);
Index: target-select.cc
===================================================================
RCS file: /cvs/src/src/gold/target-select.cc,v
retrieving revision 1.6
diff -p -u -r1.6 target-select.cc
--- target-select.cc	13 Mar 2008 21:04:21 -0000	1.6
+++ target-select.cc	25 Mar 2008 21:14:53 -0000
@@ -41,8 +41,11 @@ namespace gold
 // list.  This runs at global constructor time, so we want it to be
 // fast.
 
-Target_selector::Target_selector(int machine, int size, bool is_big_endian)
-  : machine_(machine), size_(size), is_big_endian_(is_big_endian)
+Target_selector::Target_selector(int machine, int size, bool is_big_endian,
+				 const char* bfd_name)
+  : machine_(machine), size_(size), is_big_endian_(is_big_endian),
+    bfd_name_(bfd_name), instantiated_target_(NULL)
+    
 {
   this->next_ = target_selectors;
   target_selectors = this;
@@ -77,11 +80,24 @@ select_target_by_name(const char* name)
 {
   for (Target_selector* p = target_selectors; p != NULL; p = p->next())
     {
-      Target* ret = p->recognize_by_name(name);
-      if (ret != NULL)
-	return ret;
+      const char* pname = p->bfd_name();
+      if (pname == NULL || strcmp(pname, name) == 0)
+	{
+	  Target* ret = p->recognize_by_name(name);
+	  if (ret != NULL)
+	    return ret;
+	}
     }
   return NULL;
 }
 
+// Push all the supported BFD names onto a vector.
+
+void
+supported_target_names(std::vector<const char*>* names)
+{
+  for (Target_selector* p = target_selectors; p != NULL; p = p->next())
+    p->supported_names(names);
+}
+
 } // End namespace gold.
Index: x86_64.cc
===================================================================
RCS file: /cvs/src/src/gold/x86_64.cc,v
retrieving revision 1.56
diff -p -u -r1.56 x86_64.cc
--- x86_64.cc	25 Mar 2008 18:37:16 -0000	1.56
+++ x86_64.cc	25 Mar 2008 21:14:53 -0000
@@ -2219,40 +2219,14 @@ class Target_selector_x86_64 : public Ta
 {
 public:
   Target_selector_x86_64()
-    : Target_selector(elfcpp::EM_X86_64, 64, false)
+    : Target_selector(elfcpp::EM_X86_64, 64, false, "elf64-x86-64")
   { }
 
   Target*
-  recognize(int machine, int osabi, int abiversion);
-
-  Target*
-  recognize_by_name(const char*);
-
- private:
-  Target_x86_64* target_;
+  do_instantiate_target()
+  { return new Target_x86_64(); }
 };
 
-// Recognize an x86_64 object file when we already know that the machine
-// number is EM_X86_64.
-
-Target*
-Target_selector_x86_64::recognize(int, int, int)
-{
-  if (this->target_ == NULL)
-    this->target_ = new Target_x86_64();
-  return this->target_;
-}
-
-Target*
-Target_selector_x86_64::recognize_by_name(const char* name)
-{
-  if (strcmp(name, "elf64-x86-64") != 0)
-    return NULL;
-  if (this->target_ == NULL)
-    this->target_ = new Target_x86_64();
-  return this->target_;
-}
-
 Target_selector_x86_64 target_selector_x86_64;
 
 } // End anonymous namespace.
Index: testsuite/testfile.cc
===================================================================
RCS file: /cvs/src/src/gold/testsuite/testfile.cc,v
retrieving revision 1.11
diff -p -u -r1.11 testfile.cc
--- testsuite/testfile.cc	13 Mar 2008 21:04:21 -0000	1.11
+++ testsuite/testfile.cc	25 Mar 2008 21:14:53 -0000
@@ -137,11 +137,18 @@ class Target_selector_test : public Targ
 {
  public:
   Target_selector_test()
-    : Target_selector(0xffff, size, big_endian)
+    : Target_selector(0xffff, size, big_endian, NULL)
   { }
 
   Target*
-  recognize(int, int, int)
+  do_instantiate_target()
+  {
+    gold_unreachable();
+    return NULL;
+  }
+
+  Target*
+  do_recognize(int, int, int)
   {
     if (size == 32)
       {
@@ -178,8 +185,12 @@ class Target_selector_test : public Targ
   }
 
   Target*
-  recognize_by_name(const char*)
+  do_recognize_by_name(const char*)
   { return NULL; }
+
+  void
+  do_supported_names(std::vector<const char*>*)
+  { }
 };
 
 // Register the test target selectors.  These don't need to be

Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Index: target-select.h
===================================================================
RCS file: /cvs/src/src/gold/target-select.h,v
retrieving revision 1.6
diff -p -u -r1.6 target-select.h
--- target-select.h	13 Mar 2008 21:04:21 -0000	1.6
+++ target-select.h	25 Mar 2008 21:14:53 -0000
@@ -23,6 +23,8 @@
 #ifndef GOLD_TARGET_SELECT_H
 #define GOLD_TARGET_SELECT_H
 
+#include <vector>
+
 namespace gold
 {
 
@@ -41,29 +43,42 @@ class Target_selector
  public:
   // Create a target selector for a specific machine number, size (32
   // or 64), and endianness.  The machine number can be EM_NONE to
-  // test for any machine number.
-  Target_selector(int machine, int size, bool is_big_endian);
+  // test for any machine number.  BFD_NAME is the name of the target
+  // used by the GNU linker, for backward compatibility; it may be
+  // NULL.
+  Target_selector(int machine, int size, bool is_big_endian,
+		  const char* bfd_name);
 
   virtual ~Target_selector()
   { }
 
   // If we can handle this target, return a pointer to a target
   // structure.  The size and endianness are known.
-  virtual Target*
-  recognize(int machine, int osabi, int abiversion) = 0;
+  Target*
+  recognize(int machine, int osabi, int abiversion)
+  { return this->do_recognize(machine, osabi, abiversion); }
 
   // If NAME matches the target, return a pointer to a target
   // structure.
-  virtual Target*
-  recognize_by_name(const char* name) = 0;
+  Target*
+  recognize_by_name(const char* name)
+  { return this->do_recognize_by_name(name); }
+
+  // Push all supported names onto the vector.  This is only used for
+  // help output.
+  void
+  supported_names(std::vector<const char*>* names)
+  { this->do_supported_names(names); }
 
   // Return the next Target_selector in the linked list.
   Target_selector*
   next() const
   { return this->next_; }
 
   // Return the machine number this selector is looking for, which can
   // be EM_NONE to match any machine number.
   int
   machine() const
   { return this->machine_; }
@@ -78,21 +93,88 @@ class Target_selector
   is_big_endian() const
   { return this->is_big_endian_; }
 
+  // Return the BFD name.  This may return NULL, in which case the
+  // do_recognize_by_name hook will be responsible for matching the
+  // BFD name.
+  const char*
+  bfd_name() const
+  { return this->bfd_name_; }
+
+ protected:
+  // Return an instance of the real target.  This must be implemented
+  // by the child class.
+  virtual Target*
+  do_instantiate_target() = 0;
+
+  // Recognize an object file given a machine code, size, and
+  // endianness.  When this is called we already know that they match
+  // the machine_, size_, and is_big_endian_ fields.  The child class
+  // may implement a different version of this to do additional
+  // checks, or to check for multiple machine codes if the machine_
+  // field is EM_NONE.
+  virtual Target*
+  do_recognize(int, int, int)
+  { return this->instantiate_target(); }
+
+  // Recognize a target by name.  When this is called we already know
+  // that the name matches (or that the bfd_name_ field is NULL).  The
+  // child class may implement a different version of this to
+  // recognize more than one name.
+  virtual Target*
+  do_recognize_by_name(const char*)
+  { return this->instantiate_target(); }
+
+  // Return a list of supported BFD names.  The child class may
+  // implement a different version of this to handle more than one
+  // name.
+  virtual void
+  do_supported_names(std::vector<const char*>* names)
+  {
+    gold_assert(this->bfd_name_ != NULL);
+    names->push_back(this->bfd_name_);
+  }
+
  private:
-  int machine_;
-  int size_;
-  bool is_big_endian_;
+  // Instantiate the target and return it.
+  Target*
+  instantiate_target()
+  {
+    if (this->instantiated_target_ == NULL)
+      this->instantiated_target_ = this->do_instantiate_target();
+    return this->instantiated_target_;
+  }
+
+  // ELF machine code.
+  const int machine_;
+  // Target size--32 or 64.
+  const int size_;
+  // Whether the target is big endian.
+  const bool is_big_endian_;
+  // BFD name of target, for compatibility.
+  const char* const bfd_name_;
+  // Next entry in list built at global constructor time.
   Target_selector* next_;
+  // The singleton Target structure--this points to an instance of the
+  // real implementation.
+  Target* instantiated_target_;
 };
 
 // Select the target for an ELF file.
 
-extern Target* select_target(int machine, int size, bool big_endian,
-			     int osabi, int abiversion);
+extern Target*
+select_target(int machine, int size, bool big_endian, int osabi,
+	      int abiversion);
 
 // Select a target using a BFD name.
 
-extern Target* select_target_by_name(const char* name);
+extern Target*
+select_target_by_name(const char* name);
+
+// Fill in a vector with the list of supported targets.  This returns
+// a list of BFD names.
+
+extern void
+supported_target_names(std::vector<const char*>*);
 
 } // End namespace gold.
 
@@ -41,29 +43,42 @@ class Target_selector
   { return this->next_; }
 
-  // Return the machine number this selector is looking for, which can
-  // be EM_NONE to match any machine number.
+  // Return the machine number this selector is looking for.  This can
+  // be EM_NONE to match any machine number, in which case the
+  // do_recognize hook will be responsible for matching the machine
+  // number.
   int
   machine() const